@@ -32,6 +32,9 @@ extern crate num_traits as num;
 #[macro_use]
 #[cfg(feature = "approx")]
 extern crate approx;
+#[cfg(feature = "serde")]
+#[macro_use]
+extern crate serde;
 
 use std::ops::*;
 use std::f64::consts;
@@ -44,34 +47,40 @@ use num::{Float, NumCast};
 /// Degrees are uniquely defined from 0..360.
 #[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Hash)]
 #[repr(transparent)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Deg<T>(pub T);
 /// An angular quantity measured in gons.
 ///
 /// Gons, or gradians, are uniquely defined from 0..400.
 #[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Hash)]
 #[repr(transparent)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Gon<T>(pub T);
 /// An angular quantity measured in degrees.
 ///
 /// Radians are uniquely defined from 0..2π.
 #[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Hash)]
 #[repr(transparent)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Rad<T>(pub T);
 /// An angular quantity measured in "turns", or full rotations.
 ///
 /// Turns are uniquely defined from 0..1.
 #[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Hash)]
 #[repr(transparent)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct Turns<T>(pub T);
 /// An angular quantity measured in arc minutes, which are
 /// 1/60th of a degree.
 #[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Hash)]
 #[repr(transparent)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct ArcMinutes<T>(pub T);
 /// An angular quantity measured in arc seconds, which are
 /// 1/60th of an arc minute.
 #[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Hash)]
 #[repr(transparent)]
+#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
 pub struct ArcSeconds<T>(pub T);
 
 /// Construct `Self` from an angle.
@@ -531,6 +540,21 @@ impl<T: Float> Deg<T> {
     }
 }
 
+impl<T: Float> Rad<T> {
+    pub fn pi() -> Rad<T> {
+        Rad(cast(consts::PI).unwrap())
+    }
+    pub fn pi_over_2() -> Rad<T> {
+        Rad(cast(consts::PI / 2.0).unwrap())
+    }
+    pub fn pi_over_3() -> Rad<T> {
+        Rad(cast(consts::PI / 3.0).unwrap())
+    }
+    pub fn pi_over_4() -> Rad<T> {
+        Rad(cast(consts::PI / 4.0).unwrap())
+    }
+}
+
 impl<T, U> IntoAngle<U> for T
     where U: Angle<Scalar = T::Scalar> + FromAngle<T>,
           T: Angle
@@ -571,12 +595,38 @@ impl<T: fmt::Display> fmt::Display for ArcSeconds<T> {
     }
 }
 
+/// Compute the mean of a collection of angles.
+///
+/// Note that because angles are circular, a standard summation and dividing by `len()`
+/// is _not_ valid, even if all angles are normalized.
+pub fn mean<T, Scalar, Out>(iter: T) -> Out
+    where T: IntoIterator,
+          T::Item: IntoAngle<Rad<Scalar>, OutputScalar=Scalar>,
+          Scalar: Float + AddAssign,
+          Out: Angle<Scalar=Scalar> + FromAngle<Rad<Scalar>>,
+{
+    let mut sum_of_sines: Scalar = cast(0.0).unwrap();
+    let mut sum_of_cosines: Scalar = cast(0.0).unwrap();
+
+    for angle in iter.into_iter() {
+        let intermediate_angle: Rad<Scalar> = angle.into_angle();
+        let (sin, cos) = intermediate_angle.sin_cos();
+        sum_of_sines += sin;
+        sum_of_cosines += cos;
+    }
+
+    Out::atan2(sum_of_sines, sum_of_cosines).normalize()
+}
+
 fn cast<T: NumCast, U: NumCast>(from: T) -> Option<U> {
     U::from(from)
 }
 
 #[cfg(test)]
 mod test {
+    #[cfg(feature = "serde")]
+    extern crate serde_test;
+
     use std::f64::consts;
     use std::f64;
     use super::*;
@@ -621,7 +671,7 @@ mod test {
 
             let mut a6 = Deg(123.0);
             a6 += Deg(10.0);
-            a6 += Rad(consts::PI);
+            a6 += Rad::pi();
             assert_ulps_eq!(a6.0, 313.0);
 
             let a7 = Deg(50.0);
@@ -661,7 +711,7 @@ mod test {
         assert_relative_eq!(Deg::acos(1.0), Deg(0.0));
         assert_relative_eq!(Deg::acos(0.0), Deg(90.0));
         assert_relative_eq!(Deg::acos(0.0), Deg::from(Gon(100.0)));
-        assert_relative_eq!(Rad::acos(0.0), Rad(consts::PI / 2.0));
+        assert_relative_eq!(Rad::acos(0.0), Rad::pi_over_2());
     }
 
     #[test]
@@ -745,6 +795,7 @@ mod test {
         assert_relative_eq!(Deg(100.0).interpolate_forward(&Deg(310.0), 0.5).normalize(), 
                             Deg(205.0));
         assert_relative_eq!(Gon(66.6666667).interpolate(&Deg(120.0), 0.5), Gon(100.0), epsilon=1e-6);
+        assert_relative_eq!(Rad::pi_over_2().interpolate(&Rad(0.0), 0.5), Rad::pi_over_4());
     }
 
     #[test]
@@ -778,4 +829,33 @@ mod test {
         assert_relative_eq!(Gon(50.0).reflect_x(), Gon(350.0));
         assert_relative_eq!(Gon(215.0).reflect_x(), Gon(185.0));
     }
+
+    #[test]
+    fn test_mean() {
+        assert_relative_eq!(mean(vec![Deg(280.0), Deg(10.0)].into_iter()), Deg(325.0));
+        assert_relative_eq!(mean(vec![Turns(0.5), Turns(0.0)].into_iter()), Deg(90.0));
+        assert_relative_eq!(mean([Rad(0.0), Rad(0.0)].into_iter().cloned()), Rad(0.0));
+    }
+
+    #[cfg(feature = "serde")]
+    #[test]
+    fn test_serialize() {
+        use self::serde_test::{Token, assert_tokens};
+
+        assert_tokens(&Deg(90.0), &[Token::NewtypeStruct {name: "Deg"}, Token::F64(90.0)]);
+        assert_tokens(&Rad(0.5f32), &[Token::NewtypeStruct {name: "Rad"}, Token::F32(0.5f32)]);
+        assert_tokens(&Gon(300.0), &[Token::NewtypeStruct {name: "Gon"}, Token::F64(300.0)]);
+        assert_tokens(&Turns(0.666), &[Token::NewtypeStruct {name: "Turns"}, Token::F64(0.666)]);
+    }
+
+    #[cfg(feature = "serde")]
+    #[test]
+    fn test_deserialize() {
+        use self::serde_test::{Token, assert_de_tokens};
+
+        assert_de_tokens(&Deg(90.0), &[Token::NewtypeStruct {name: "Deg"}, Token::F64(90.0)]);
+        assert_de_tokens(&Rad(0.5f32), &[Token::NewtypeStruct {name: "Rad"}, Token::F32(0.5f32)]);
+        assert_de_tokens(&Gon(300.0), &[Token::NewtypeStruct {name: "Gon"}, Token::F64(300.0)]);
+        assert_de_tokens(&Turns(0.666), &[Token::NewtypeStruct {name: "Turns"}, Token::F64(0.666)]);
+    }
 }
